function EditUserRequest(){
  this.version=SETTINGS.DEFAULT_VERSION;
  this.lang=SETTINGS.DEFAULT_LANG;
  this.userName="";
  this.contextName="VERIZON";
  this.groupName="";
  this.firstName="";
  this.middleName="";
  this.lastName="";
  this.companyName="";
  this.faxNumber="";
  this.primaryPhoneNumber="";
  this.secondaryPhoneNumber="";
  this.primaryEmail="";
  this.secondaryEmail="";
  this.addressType="BUSINESS";
  this.streetAddress1="";
  this.streetAddress2="";
  this.streetAddress3="";
  this.streetAddress4="";
  this.city="";
  this.postalCode="";
  this.state="";
  this.country="";
  this.isActive=true;
  this.isLocked=false;
  this.authToken="";
  this.userVersionId="1";
  this.adminUserName="";
  this.adminContextName="";
  this.attributes=new Array();
}

var validatorEditUserForm = jQuery("#editUserForm").validate({
    onfocusout:false,
    onkeyup:false,
	rules: {
		email: {
			required: true,
			validEmail: true
		},
		firstName: {
			required: true
		},
		lastName: {
			required: true
		},

		address1: {
			required: true
		},
		city: {
			required: true
		},
		state: {
			required: true
		},
		zip: {
			required: true
		},
		userName: {
			required: true,
			validUserName: true
		},
		group: {
			required: true
		}
	},
	errorElement: "label" ,
	errorPlacement: function(error, element) {
		error.insertAfter(element);
	},
	messages: {
		email: {
			required: MESSAGES.ERROR.ERROR_EMAIL_REQ_MSG,
			validEmail: MESSAGES.ERROR.ERROR_EMAIL_NOT_VZW
		},
		firstName: {
			required: MESSAGES.ERROR.ERROR_FIRST_NAME_REQ_MSG
		},
		lastName: {
			required: MESSAGES.ERROR.ERROR_LAST_NAME_REQ_MSG
		},
		address1: {
			required: MESSAGES.ERROR.ERROR_ADDRESS_REQ_MSG
		},
		city: {
			required: MESSAGES.ERROR.ERROR_CITY_REQ_MSG
		},
		state: {
			required: MESSAGES.ERROR.ERROR_STATE_REQ_MSG
		},
		zip: {
			required: MESSAGES.ERROR.ERROR_ZIP_REQ_MSG
		},
		userName: {
			required: MESSAGES.ERROR.USERNAME_REQUIRED,
			validUserName: MESSAGES.ERROR.USERNAME_INVALID
		},
		group: {
			required: MESSAGES.ERROR.ERROR_AUTH_GROUP_REQ_MSG
		}
	}
});

function loadUserDetails(user) {
	jQuery('#email').val(getNonNullValue(user.consumerDemog["primaryEmail"]));
	jQuery('#firstName').val(getNonNullValue(user.consumerDemog["firstName"]));
	jQuery('#lastName').val(getNonNullValue(user.consumerDemog["lastName"]));
	jQuery('#address1').val(getNonNullValue(user.consumerDemog.addressList[0].streetAddress1));
	jQuery('#address2').val(getNonNullValue(user.consumerDemog.addressList[0].streetAddress2));
	jQuery('#city').val(getNonNullValue(user.consumerDemog.addressList[0].city));
	jQuery('#state').val(getNonNullValue(user.consumerDemog.addressList[0].state.toUpperCase()));
	jQuery('#zip').val(getNonNullValue(user.consumerDemog.addressList[0].postalCode));
	jQuery('#userName').val(getNonNullValue(user["userName"]));
	jQuery('#group').val(getNonNullValue(user["groupName"]));

	if(getNonNullValue(user["locked"]) == "YES"){
		jQuery('#statusLocked').prop( "checked", true );
	}
	else if(getNonNullValue(user["status"]) != "ACTIVE"){
		jQuery('#statusDisabled').prop( "checked", true );
	}
	else if(getNonNullValue(user["status"]) != "INACTIVE"){
		jQuery('#statusEnabled').prop( "checked", true );
	}
}

function handleRespOnSaveSuccess( data, statusCode, xhr ) {
	//Triggered after a successful response from server
	if(data.status == "SUCCESS" || data.status == "SUCESS" ) {
		loadUser(data.userName);
		showSuccessMsg("Saved Successfully!");
	}
}

function handleRespOnSuccess( data, statusCode, xhr ) {
	//Triggered after a successful response from server
	if(data.status == "SUCCESS" || data.status == "SUCESS" ) {
		loadUserDetails(data.userVO);
		saveTableResultsInStore(data);
	}
}

function showErrorMsg(message) {
	jQuery('#userSuccessPanel').hide();
	jQuery('#userErrorPanel').show();
	jQuery('#userErrorMsg').html(message);
}

function showSuccessMsg(message) {
	jQuery('#userSuccessPanel').show();
	jQuery('#userErrorPanel').hide();
	jQuery('#userSuccessMsg').html(message);
}

function handleRespOnError( jqXHR, errorType, exception ) {

	var message = MESSAGES.ERROR.TECHNICAL_ERROR;
	if(hasUserAuthenticated(jqXHR, false) == false  ) {
		message = MESSAGES.STATUS.USER_SESSION_EXPIRED;
	} else if(hasUserAuthorized(jqXHR) == false  ) {
		message = MESSAGES.STATUS.RESOURCE_ACCESS_DENIED;
	}
	try{
		if(jqXHR.responseText !== 'undefined' && jqXHR.responseText != null)
		{
			var data = JSON.parse(jqXHR.responseText);
			if (typeof (console) != 'undefined' && console != null) {
				console.log("Error");
				console.log(data.status);
				console.log(data.errorCode);
			}
			if(data.status != null && data.status == 'ERROR')
			{
				if(data.errorCode ==='VZ_AS_USER_CONFLICT')
				{
					message=MESSAGES.ERROR.USER_ALREADY_EXISTS;
				}
				else if(data.errorCode ==='VALIDATION_ERROR')
				{
					message+=MESSAGES.ERROR.VALIDATION_ERROR;
					var str = data.errorDetails.split(",");
					var i=0;
					message+="<ul>";
					for(i = 0; i < str.length; i++){
						message+="<li>" + str[i].replace(":invalid","") + "</li>";
					}
					message+="</ul>";
				}
			}
		}
	} catch(e) {
		console.log("Exception occured");
	}
	showErrorMsg(message);
}

function loadUser(userName){
	var ajaxRequest = {
		type: 'GET',
		url: '',
		datType: 'json',
		async : true,
		cache : false,
		contentType: SETTINGS.CONTENT_TYPE,
		complete : function(){
			jQuery.unblockUI();
		},
		error: handleRespOnError,
		success: handleRespOnSuccess
	};


	//Set Ajax Request Data
	ajaxRequest.url=URLS.VIEW_USER_REST_SERVICE_URL+'/'+SETTINGS.DEFAULT_VERSION+'/'+SETTINGS.DEFAULT_LANG+'/'+ userName +'/VERIZON';
	var AjaxResult = jQuery.ajax(ajaxRequest);
	var tblResData =loadTableResultsFromStore();
	if(tblResData != null) {
		loadUserDetails(tblResData.userVO);
	}
}

jQuery(document).ready(function() {

	checkUserSession(true);
	welcomeUser();
	displayAdminNav();
	loadUser(GetURLParameter('userName'));


	//Prepare for saving
	jQuery.validator.addMethod("validUserName", function(value, element) {
    	return this.optional(element) || VALIDATIONS.REGEX.USERNAME_REGEX.test(value);
    }, MESSAGES.ERROR.USERNAME_INVALID);

	jQuery.validator.addMethod("validEmail", function(value, element) {
    	return VALIDATIONS.REGEX.VZW_EMAIL.test(value);
    }, MESSAGES.ERROR.ERROR_EMAIL_NOT_VZW);

	jQuery('#saveButton').click(function() {
		checkUserSession(true);
		jQuery('#email').val(jQuery.trim(jQuery('#email').val()));
		jQuery('#firstName').val(jQuery.trim(jQuery('#firstName').val()));
		jQuery('#lastName').val(jQuery.trim(jQuery('#lastName').val()));
		jQuery('#address1').val(jQuery.trim(jQuery('#address1').val()));
		jQuery('#address2').val(jQuery.trim(jQuery('#address2').val()));
		jQuery('#city').val(jQuery.trim(jQuery('#city').val()));
		jQuery('#zip').val(jQuery.trim(jQuery('#zip').val()));
		jQuery('#userName').val(jQuery.trim(jQuery('#userName').val()));

		var ajaxRequest = {
			type: 'POST',
			url: URLS.EDIT_USER_REST_SERVICE_URL,
			data: '{}',
			dataType: 'json',
			async : true,
			cache : false,
			contentType: SETTINGS.CONTENT_TYPE,
			beforeSend: function() {
				jQuery.blockUI({message: MESSAGES.STATUS.AJAX_LOADING});
	        },
	        complete : function(){
				jQuery.unblockUI();
	        },
			error: handleRespOnError,
			success: handleRespOnSaveSuccess
		};

		if(validatorEditUserForm.form()) {
			editUserRequest= new EditUserRequest();

			if(jQuery('#email').val()!=""){
				editUserRequest.primaryEmail =jQuery('#email').val();
			}
			if(jQuery('#firstName').val()!=""){
				editUserRequest.firstName =jQuery('#firstName').val();
			}
			if(jQuery('#lastName').val()!=""){
				editUserRequest.lastName =jQuery('#lastName').val();
			}
			if(jQuery('#address1').val()!=""){
				editUserRequest.streetAddress1 =jQuery('#address1').val();
			}
			if(jQuery('#address2').val()!=""){
				editUserRequest.streetAddress2 =jQuery('#address2').val();
			}
			if(jQuery('#city').val()!=""){
				editUserRequest.city =jQuery('#city').val();
			}
			if(jQuery('#state').val()!=""){
				editUserRequest.state =jQuery('#state').val();
			}
			if(jQuery('#zip').val()!=""){
				editUserRequest.postalCode =jQuery('#zip').val();
			}
			if(jQuery('#userName').val()!=""){
				editUserRequest.userName=jQuery('#userName').val();
			}
			if(jQuery('#group').val()!=""){
				editUserRequest.groupName=jQuery('#group').val();
			}

			if(jQuery('#statusLocked').prop( "checked")==true ){
				editUserRequest.isActive=true;
				editUserRequest.isLocked=true;
			}else if(jQuery('#statusDisabled').prop( "checked")==true ){
				editUserRequest.isActive=false;
				editUserRequest.isLocked=false;
			}else if(jQuery('#statusEnabled').prop( "checked")==true ){
				editUserRequest.isActive=true;
				editUserRequest.isLocked=false;
			}

			editUserRequest.authToken=getUserAuthToken();
			editUserRequest.adminUserName=getuserName();
			editUserRequest.adminContextName=getUserContext();

			//Set Ajax Request Data
			ajaxRequest.data=JSON.stringify(editUserRequest);
			var AjaxResult = jQuery.ajax(ajaxRequest);
		} else {
			return false;
		}
			return false;
	}); 

});